/* eslint no-param-reassign: 0 */
import React from 'react';
import { identity, isFunction, isNil, isObject, isArray, isNumber, isString, toString, } from 'lodash';
import accounting from 'accounting';
import defineDisplay from './defineDisplay';
import { warning, invariant } from './kit';
import { timestampToString } from '../utils';
import { getMutable } from '../runtime';
/**
 * 用于从数据中获取需要的值
 *
 * @example
 * extract({ fn: (value, record) => record.a + record.b })(undefined, { a: 1, b: 2 })
 * // => 3
 */
export const extract = defineDisplay('extract', (value, params, ctx) => {
    const { record } = ctx;
    invariant(isFunction(params.fn), '`extractDisplay 参数 fn 必须为函数`');
    return params.fn(value, record);
}).curry();
/**
 * 显示不换行
 *
 * @example
 * nowrap()('text')
 * // => <div style={ 'white-space': 'nowrap' }>text</div>
 */
export const nowrap = defineDisplay('nowrap', (value, { inline }) => {
    const tagName = inline ? 'span' : 'div';
    return React.createElement(tagName, {
        style: { whiteSpace: 'nowrap' },
    }, value);
});
/**
 * 格式化常量数据
 *
 * @example
 * const map = { m: '男', f: '女' };
 * consts({ map })('m');
 * // => '男'
 */
export const consts = defineDisplay('consts', {
    defaultValue: identity,
}, 'map', (value, { defaultValue, map, array, }) => {
    function getConstValue(v, mapper) {
        if (isNumber(v)) {
            v = toString(v);
        }
        const exist = Object.prototype.hasOwnProperty.call(mapper, v);
        if (!exist) {
            let returnValue = defaultValue;
            if (isFunction(defaultValue)) {
                returnValue = defaultValue(v);
            }
            if (returnValue === undefined) {
                returnValue = v;
            }
            return returnValue;
        }
        return mapper[v];
    }
    let resolvedMapper;
    if (!isObject(map)) {
        warning(map == null, '`constsDisplay` 的参数 map 需为对象');
        resolvedMapper = {};
    }
    else if (Array.isArray(map)) {
        resolvedMapper = map.reduce((p, item) => {
            if ('k' in item) {
                p[item.k] = item.v;
            }
            else {
                p[item.value] = item.label;
            }
            return p;
        }, {});
    }
    else {
        resolvedMapper = map;
    }
    if (array) {
        let arrValue;
        if (isString(value)) {
            arrValue = value.split(',');
        }
        else if (isArray(value)) {
            arrValue = value;
        }
        if (arrValue) {
            return arrValue.map((v) => getConstValue(v, resolvedMapper)).join(',');
        }
    }
    return getConstValue(value, resolvedMapper);
}).curry();
/**
 * 若数据为 null 或 undefined，格式化为默认值
 *
 * @return {Function}
 * @example
 * defaults({ defaultValue: '-' })(null)
 * // => '-'
 */
export const defaults = defineDisplay('defaults', {
    defaultValue: '',
    formatString: false,
}, (value, { defaultValue, formatString }) => {
    if (isNil(value)) {
        return defaultValue;
    }
    return formatString ? toString(value) : value;
}).curry();
/**
 * 格式化时间
 *
 * @return {Function}
 * @example
 * formatTime()(1471564800000); // '2016-08-19 08:00:00'
 */
export const formatTime = defineDisplay('formatTime', {
    format: getMutable('TIME_FORMAT'),
    formatNaN: identity,
}, 'format', (value, { format, formatNaN }) => {
    if (toString(value) === '') {
        return '';
    }
    return timestampToString(value, format, formatNaN);
}).curry();
/**
 * 格式化日期
 *
 * @return {Function}
 * @example
 * formatDate()(1471564800000); // '2016-08-19'
 */
export const formatDate = defineDisplay('formatDate', {
    format: getMutable('DATE_FORMAT'),
    formatNaN: identity,
}, 'format', (value, { format, formatNaN }) => {
    if (toString(value) === '') {
        return '';
    }
    return timestampToString(value, format, formatNaN);
}).curry();
/**
 * 格式化数值成整数
 * @param  {Object} conf
 * @param  {Function} [conf.formatNaN] - 值解析为 NaN 时输出的函数
 * @param  {Function} [conf.preprocess] - 预处理值
 * @param  {Function} [conf.format] - 格式化输出，参考 accounting 文档
 * @return {String}
 */
export const integer = defineDisplay('integer', {
    // 格式
    format: undefined,
}, (value, { formatNaN, preprocess, format }) => {
    if (isFunction(preprocess)) {
        value = preprocess(value);
    }
    const numValue = +value;
    if (Number.isNaN(numValue)) {
        return isFunction(formatNaN) ? formatNaN(numValue) : '';
    }
    return accounting.formatNumber(value, format);
}).curry();
/**
 * 格式化金额，金额单位为元
 * @param  {Object} conf
 * @param  {Function} [conf.formatNaN] - 值解析为 NaN 时输出的函数
 * @param  {Function} [conf.preprocess] - 预处理值
 * @param  {Number} [conf.precision=2] - 小数精度，默认为 2
 * @param  {String} [conf.symbol=''] - 金额单位，默认为空
 * @param  {Function} [conf.format] - 格式化输出，参考 accounting 文档
 * @return {String}
 */
export const money = defineDisplay('money', {
    // 小数位数
    precision: 2,
    // 单位
    symbol: '',
    // 格式
    format: undefined,
}, (value, { formatNaN, preprocess, precision, symbol, format }) => {
    if (isFunction(preprocess)) {
        value = preprocess(value);
    }
    else if (getMutable('MONEY_PRESET_UNIT_CENT')) {
        value /= 100;
    }
    const numValue = +value;
    if (Number.isNaN(numValue)) {
        return isFunction(formatNaN) ? formatNaN(numValue) : '';
    }
    return accounting.formatMoney(value, {
        precision,
        symbol,
        format,
    });
}).curry();
export default {
    consts,
    defaults,
    formatDate,
    formatTime,
    integer,
    money,
};
