import { get, toLower, uniqueId } from 'lodash';
import warning from 'warning';
import { emit, getMutable } from '../runtime/internal';
import { ensureFunction, extendUrlQuery } from '../utils';
import DSError from './Error';
import normalizeDsArguments from './logics/normalizeDsArguments';
import resolveCustomHeaders from './logics/resolveCustomHeaders';
import resolveRequestPath from './logics/resolveRequestPath';
export const dsSymbol = '__BF_CORE_SYMBOL_CALL_API__';
export const defaultGetDataFromResponse = (response) => get(response, 'data');
/**
 * @deprecated
 */
function deprecatedOnRequestStart(requestMark) {
    const onRequestStart = getMutable('DS_ON_REQUEST_START');
    if (typeof onRequestStart === 'function') {
        warning(false, `[deprecated] config('DS_ON_REQUEST_START', handler) 已废弃，请使用 on('DS_REQUEST_START', handler) 替换`);
        onRequestStart(requestMark);
    }
}
/**
 * @deprecated
 */
function deprecatedOnRequestEnd(e, responseData, requestMark) {
    const onRequestEnd = getMutable('DS_ON_REQUEST_END');
    if (typeof onRequestEnd === 'function') {
        warning(false, `[deprecated] config('DS_ON_REQUEST_END', handler) 已废弃，请使用 on('DS_REQUEST_END', handler) 替换`);
        onRequestEnd(e, responseData, requestMark);
    }
}
async function ds(v, ...args) {
    const { dsPath, params, config } = normalizeDsArguments(v, ...args);
    const getDataFromResponse = ensureFunction(get(config, 'getDataFromResponse'), defaultGetDataFromResponse);
    const fetcherInit = get(config, 'fetcherInit');
    const method = toLower(get(config, 'method') || 'post');
    let url = resolveRequestPath(dsPath);
    let body = JSON.stringify(params);
    // Request with GET/HEAD method cannot have body
    if (['get', 'head'].includes(method)) {
        url = extendUrlQuery(url, params);
        body = undefined;
    }
    const requestId = `${Date.now()}${uniqueId()}`;
    const requestMark = {
        url,
        dsPath,
        params,
        id: requestId,
    };
    emit('DS_REQUEST_START', requestMark);
    deprecatedOnRequestStart(requestMark);
    return window
        .fetch(url, Object.assign({ method, headers: Object.assign({ 'Content-Type': 'application/json' }, resolveCustomHeaders(dsPath, url)), body, credentials: 'include', mode: 'cors' }, fetcherInit))
        .catch((e) => {
        if (e instanceof TypeError) {
            console.error(e);
        }
        return Promise.reject(new DSError(e.message, {
            isNetworkError: true,
            isHttpError: true,
        }));
    })
        .then((response) => {
        if (!response.ok) {
            const message = `[${response.status}] ${response.statusText}`;
            return Promise.reject(new DSError(message, {
                isNetworkError: false,
                isHttpError: true,
                httpStatus: response.status,
            }));
        }
        const forDescripeErrorResp = response.clone();
        return response
            .json()
            .catch((e) => {
            return forDescripeErrorResp
                .text()
                .catch(() => {
                return '';
            })
                .then((text) => {
                return Promise.reject(new DSError('请求失败：不能解析成有效 JSON', {
                    httpStatus: response.status,
                    isParsedError: true,
                    responseBodyText: text,
                }));
            });
        })
            .then((json) => {
            if (json && typeof json === 'object' && 'status' in json) {
                return json;
            }
            return Promise.reject(new DSError('请求失败：status 不在返回数据中', {
                httpStatus: response.status,
                isParsedError: true,
                responseBodyText: JSON.stringify(json),
            }));
        });
    })
        .then((data) => {
        const status = +data.status;
        if (status !== 0) {
            let errMsg = '';
            if (status === 401) {
                const noAuthCallback = getMutable('DS_UNAUTHORIZED_CALLBACK');
                if (typeof noAuthCallback === 'function') {
                    errMsg = noAuthCallback(data);
                }
            }
            else if (status === 403) {
                errMsg = '没有权限';
            }
            else if (data.message) {
                errMsg = data.message;
            }
            else {
                errMsg = '请求失败';
            }
            return Promise.reject(new DSError(errMsg, {
                httpStatus: 200,
                status,
            }));
        }
        const responseData = getDataFromResponse(data);
        emit('DS_REQUEST_END', undefined, responseData, requestMark);
        deprecatedOnRequestEnd(undefined, responseData, requestMark);
        return responseData;
    })
        .catch((e) => {
        emit('DS_REQUEST_END', e, undefined, requestMark);
        deprecatedOnRequestEnd(e, undefined, requestMark);
        return Promise.reject(e);
    });
}
ds[dsSymbol] = true;
export default ds;
/**
 * 检测函数是否通过 ds() 创建的
 *
 * @param func
 * @return
 */
export function isDs(func) {
    return func && Object.prototype.hasOwnProperty.call(func, dsSymbol);
}
