"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.money = exports.integer = exports.formatDate = exports.formatTime = exports.defaults = exports.consts = exports.nowrap = exports.extract = void 0;
/* eslint no-param-reassign: 0 */
const react_1 = __importDefault(require("react"));
const lodash_1 = require("lodash");
const accounting_1 = __importDefault(require("accounting"));
const defineDisplay_1 = __importDefault(require("./defineDisplay"));
const kit_1 = require("./kit");
const utils_1 = require("../utils");
const runtime_1 = require("../runtime");
/**
 * 用于从数据中获取需要的值
 *
 * @example
 * extract({ fn: (value, record) => record.a + record.b })(undefined, { a: 1, b: 2 })
 * // => 3
 */
exports.extract = defineDisplay_1.default('extract', (value, params, ctx) => {
    const { record } = ctx;
    kit_1.invariant(lodash_1.isFunction(params.fn), '`extractDisplay 参数 fn 必须为函数`');
    return params.fn(value, record);
}).curry();
/**
 * 显示不换行
 *
 * @example
 * nowrap()('text')
 * // => <div style={ 'white-space': 'nowrap' }>text</div>
 */
exports.nowrap = defineDisplay_1.default('nowrap', (value, { inline }) => {
    const tagName = inline ? 'span' : 'div';
    return react_1.default.createElement(tagName, {
        style: { whiteSpace: 'nowrap' },
    }, value);
});
/**
 * 格式化常量数据
 *
 * @example
 * const map = { m: '男', f: '女' };
 * consts({ map })('m');
 * // => '男'
 */
exports.consts = defineDisplay_1.default('consts', {
    defaultValue: lodash_1.identity,
}, 'map', (value, { defaultValue, map, array, }) => {
    function getConstValue(v, mapper) {
        if (lodash_1.isNumber(v)) {
            v = lodash_1.toString(v);
        }
        const exist = Object.prototype.hasOwnProperty.call(mapper, v);
        if (!exist) {
            let returnValue = defaultValue;
            if (lodash_1.isFunction(defaultValue)) {
                returnValue = defaultValue(v);
            }
            if (returnValue === undefined) {
                returnValue = v;
            }
            return returnValue;
        }
        return mapper[v];
    }
    let resolvedMapper;
    if (!lodash_1.isObject(map)) {
        kit_1.warning(map == null, '`constsDisplay` 的参数 map 需为对象');
        resolvedMapper = {};
    }
    else if (Array.isArray(map)) {
        resolvedMapper = map.reduce((p, item) => {
            if ('k' in item) {
                p[item.k] = item.v;
            }
            else {
                p[item.value] = item.label;
            }
            return p;
        }, {});
    }
    else {
        resolvedMapper = map;
    }
    if (array) {
        let arrValue;
        if (lodash_1.isString(value)) {
            arrValue = value.split(',');
        }
        else if (lodash_1.isArray(value)) {
            arrValue = value;
        }
        if (arrValue) {
            return arrValue.map((v) => getConstValue(v, resolvedMapper)).join(',');
        }
    }
    return getConstValue(value, resolvedMapper);
}).curry();
/**
 * 若数据为 null 或 undefined，格式化为默认值
 *
 * @return {Function}
 * @example
 * defaults({ defaultValue: '-' })(null)
 * // => '-'
 */
exports.defaults = defineDisplay_1.default('defaults', {
    defaultValue: '',
    formatString: false,
}, (value, { defaultValue, formatString }) => {
    if (lodash_1.isNil(value)) {
        return defaultValue;
    }
    return formatString ? lodash_1.toString(value) : value;
}).curry();
/**
 * 格式化时间
 *
 * @return {Function}
 * @example
 * formatTime()(1471564800000); // '2016-08-19 08:00:00'
 */
exports.formatTime = defineDisplay_1.default('formatTime', {
    format: runtime_1.getMutable('TIME_FORMAT'),
    formatNaN: lodash_1.identity,
}, 'format', (value, { format, formatNaN }) => {
    if (lodash_1.toString(value) === '') {
        return '';
    }
    return utils_1.timestampToString(value, format, formatNaN);
}).curry();
/**
 * 格式化日期
 *
 * @return {Function}
 * @example
 * formatDate()(1471564800000); // '2016-08-19'
 */
exports.formatDate = defineDisplay_1.default('formatDate', {
    format: runtime_1.getMutable('DATE_FORMAT'),
    formatNaN: lodash_1.identity,
}, 'format', (value, { format, formatNaN }) => {
    if (lodash_1.toString(value) === '') {
        return '';
    }
    return utils_1.timestampToString(value, format, formatNaN);
}).curry();
/**
 * 格式化数值成整数
 * @param  {Object} conf
 * @param  {Function} [conf.formatNaN] - 值解析为 NaN 时输出的函数
 * @param  {Function} [conf.preprocess] - 预处理值
 * @param  {Function} [conf.format] - 格式化输出，参考 accounting 文档
 * @return {String}
 */
exports.integer = defineDisplay_1.default('integer', {
    // 格式
    format: undefined,
}, (value, { formatNaN, preprocess, format }) => {
    if (lodash_1.isFunction(preprocess)) {
        value = preprocess(value);
    }
    const numValue = +value;
    if (Number.isNaN(numValue)) {
        return lodash_1.isFunction(formatNaN) ? formatNaN(numValue) : '';
    }
    return accounting_1.default.formatNumber(value, format);
}).curry();
/**
 * 格式化金额，金额单位为元
 * @param  {Object} conf
 * @param  {Function} [conf.formatNaN] - 值解析为 NaN 时输出的函数
 * @param  {Function} [conf.preprocess] - 预处理值
 * @param  {Number} [conf.precision=2] - 小数精度，默认为 2
 * @param  {String} [conf.symbol=''] - 金额单位，默认为空
 * @param  {Function} [conf.format] - 格式化输出，参考 accounting 文档
 * @return {String}
 */
exports.money = defineDisplay_1.default('money', {
    // 小数位数
    precision: 2,
    // 单位
    symbol: '',
    // 格式
    format: undefined,
}, (value, { formatNaN, preprocess, precision, symbol, format }) => {
    if (lodash_1.isFunction(preprocess)) {
        value = preprocess(value);
    }
    else if (runtime_1.getMutable('MONEY_PRESET_UNIT_CENT')) {
        value /= 100;
    }
    const numValue = +value;
    if (Number.isNaN(numValue)) {
        return lodash_1.isFunction(formatNaN) ? formatNaN(numValue) : '';
    }
    return accounting_1.default.formatMoney(value, {
        precision,
        symbol,
        format,
    });
}).curry();
exports.default = {
    consts: exports.consts,
    defaults: exports.defaults,
    formatDate: exports.formatDate,
    formatTime: exports.formatTime,
    integer: exports.integer,
    money: exports.money,
};
