"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isDs = exports.defaultGetDataFromResponse = exports.dsSymbol = void 0;
const lodash_1 = require("lodash");
const warning_1 = __importDefault(require("warning"));
const internal_1 = require("../runtime/internal");
const utils_1 = require("../utils");
const Error_1 = __importDefault(require("./Error"));
const normalizeDsArguments_1 = __importDefault(require("./logics/normalizeDsArguments"));
const resolveCustomHeaders_1 = __importDefault(require("./logics/resolveCustomHeaders"));
const resolveRequestPath_1 = __importDefault(require("./logics/resolveRequestPath"));
exports.dsSymbol = '__BF_CORE_SYMBOL_CALL_API__';
exports.defaultGetDataFromResponse = (response) => lodash_1.get(response, 'data');
/**
 * @deprecated
 */
function deprecatedOnRequestStart(requestMark) {
    const onRequestStart = internal_1.getMutable('DS_ON_REQUEST_START');
    if (typeof onRequestStart === 'function') {
        warning_1.default(false, `[deprecated] config('DS_ON_REQUEST_START', handler) 已废弃，请使用 on('DS_REQUEST_START', handler) 替换`);
        onRequestStart(requestMark);
    }
}
/**
 * @deprecated
 */
function deprecatedOnRequestEnd(e, responseData, requestMark) {
    const onRequestEnd = internal_1.getMutable('DS_ON_REQUEST_END');
    if (typeof onRequestEnd === 'function') {
        warning_1.default(false, `[deprecated] config('DS_ON_REQUEST_END', handler) 已废弃，请使用 on('DS_REQUEST_END', handler) 替换`);
        onRequestEnd(e, responseData, requestMark);
    }
}
async function ds(v, ...args) {
    const { dsPath, params, config } = normalizeDsArguments_1.default(v, ...args);
    const getDataFromResponse = utils_1.ensureFunction(lodash_1.get(config, 'getDataFromResponse'), exports.defaultGetDataFromResponse);
    const fetcherInit = lodash_1.get(config, 'fetcherInit');
    const method = lodash_1.toLower(lodash_1.get(config, 'method') || 'post');
    let url = resolveRequestPath_1.default(dsPath);
    let body = JSON.stringify(params);
    // Request with GET/HEAD method cannot have body
    if (['get', 'head'].includes(method)) {
        url = utils_1.extendUrlQuery(url, params);
        body = undefined;
    }
    const requestId = `${Date.now()}${lodash_1.uniqueId()}`;
    const requestMark = {
        url,
        dsPath,
        params,
        id: requestId,
    };
    internal_1.emit('DS_REQUEST_START', requestMark);
    deprecatedOnRequestStart(requestMark);
    return window
        .fetch(url, Object.assign({ method, headers: Object.assign({ 'Content-Type': 'application/json' }, resolveCustomHeaders_1.default(dsPath, url)), body, credentials: 'include', mode: 'cors' }, fetcherInit))
        .catch((e) => {
        if (e instanceof TypeError) {
            console.error(e);
        }
        return Promise.reject(new Error_1.default(e.message, {
            isNetworkError: true,
            isHttpError: true,
        }));
    })
        .then((response) => {
        if (!response.ok) {
            const message = `[${response.status}] ${response.statusText}`;
            return Promise.reject(new Error_1.default(message, {
                isNetworkError: false,
                isHttpError: true,
                httpStatus: response.status,
            }));
        }
        const forDescripeErrorResp = response.clone();
        return response
            .json()
            .catch((e) => {
            return forDescripeErrorResp
                .text()
                .catch(() => {
                return '';
            })
                .then((text) => {
                return Promise.reject(new Error_1.default('请求失败：不能解析成有效 JSON', {
                    httpStatus: response.status,
                    isParsedError: true,
                    responseBodyText: text,
                }));
            });
        })
            .then((json) => {
            if (json && typeof json === 'object' && 'status' in json) {
                return json;
            }
            return Promise.reject(new Error_1.default('请求失败：status 不在返回数据中', {
                httpStatus: response.status,
                isParsedError: true,
                responseBodyText: JSON.stringify(json),
            }));
        });
    })
        .then((data) => {
        const status = +data.status;
        if (status !== 0) {
            let errMsg = '';
            if (status === 401) {
                const noAuthCallback = internal_1.getMutable('DS_UNAUTHORIZED_CALLBACK');
                if (typeof noAuthCallback === 'function') {
                    errMsg = noAuthCallback(data);
                }
            }
            else if (status === 403) {
                errMsg = '没有权限';
            }
            else if (data.message) {
                errMsg = data.message;
            }
            else {
                errMsg = '请求失败';
            }
            return Promise.reject(new Error_1.default(errMsg, {
                httpStatus: 200,
                status,
            }));
        }
        const responseData = getDataFromResponse(data);
        internal_1.emit('DS_REQUEST_END', undefined, responseData, requestMark);
        deprecatedOnRequestEnd(undefined, responseData, requestMark);
        return responseData;
    })
        .catch((e) => {
        internal_1.emit('DS_REQUEST_END', e, undefined, requestMark);
        deprecatedOnRequestEnd(e, undefined, requestMark);
        return Promise.reject(e);
    });
}
ds[exports.dsSymbol] = true;
exports.default = ds;
/**
 * 检测函数是否通过 ds() 创建的
 *
 * @param func
 * @return
 */
function isDs(func) {
    return func && Object.prototype.hasOwnProperty.call(func, exports.dsSymbol);
}
exports.isDs = isDs;
